package com.jz.dm.controller;

import com.jz.common.utils.Result;
import com.jz.dm.models.req.api.ApiInterfaceDetailReq;
import com.jz.dm.models.req.api.ApiInterfaceInfoListReq;
import com.jz.dm.models.resp.SendDataReq;
import com.jz.dm.service.ApiInterfaceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.controller
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiInterfaceController
 * @DATE: 2021-1-14/15:29
 * @DAY_NAME_SHORT: 周四
 * @Description:
 **/
@RestController
@RequestMapping("api/interface")
@Api(tags = "API接口Controller")
public class ApiInterfaceController {

    @Autowired
    private ApiInterfaceService apiInterfaceService;

    @ApiOperation("服务管理---API计量(已调用)")
    @PostMapping(value = "/listCallApi")
    public Mono<Result> getApiInterfaceCallList(@RequestBody @Valid ApiInterfaceInfoListReq req) {
        return Mono.fromSupplier(() -> Result.of_success(apiInterfaceService.listCallApiInterface(req)));
    }

    @ApiOperation("服务管理---API计量(未调用)")
    @PostMapping(value = "/listUnCallApi")
    public Mono<Result> getApiInterfaceList(@RequestBody @Valid ApiInterfaceInfoListReq req) {
        return Mono.fromSupplier(() -> Result.of_success(apiInterfaceService.listUnCallApiInterface(req)));
    }
    @ApiOperation("Api计量")
    @GetMapping(value = "/countAPiCallStat")
    public Mono<Result> countAPiCallStat(@RequestParam(name = "date") String date) {
        return Mono.fromSupplier(() -> apiInterfaceService.countAPiCallStat(date));
    }
    @ApiOperation("服务开发----API列表")
    @PostMapping(value = "/listServerApplyApi")
    public Mono<Result> getServerApplyApiInterfaceList(@RequestBody @Valid ApiInterfaceInfoListReq req) {
        return Mono.fromSupplier(() -> apiInterfaceService.listServerApplyApi(req));
    }

    @ApiOperation("API详情查询")
    @PostMapping(value = "/getApiDetail")
    public Mono<Result> getApiDetail(@RequestBody @Valid ApiInterfaceDetailReq req) {
        return Mono.fromSupplier(() -> apiInterfaceService.getApiInterfaceDetail(req));
    }

    @ApiOperation("数据银行API删除")
    @PostMapping(value = "/delDataBankApiInterface")
    public Mono<Result> delDataBankApiInterface(@RequestParam (name="apiKey")String apiKey) {
        return Mono.fromSupplier(() -> apiInterfaceService.delApiInterface(apiKey));
    }

    @ApiOperation("DMP-API删除/确认提示接口")
    @GetMapping(value = "/delDMPApiInterface")
    public Mono<Result> delDMPApiInterface(@RequestParam (name="type")String type,
                                           @RequestParam (name="apiKey")String apiKey) {
        return Mono.fromSupplier(() -> apiInterfaceService.dmpDeleteAuth(type,apiKey));
    }


    @ApiOperation("数据修改发送状态接口")
    @GetMapping(value = "/updateSendStatus")
    public Mono<Result> updateApiSendStatus(@RequestParam (name="apiKey")String apiKey) {
        return Mono.fromSupplier(() -> apiInterfaceService.updateApiSendStatus(apiKey));
    }

    @ApiOperation("已发送到数据银行API列表")
    @PostMapping(value = "/listSendDataBank")
    public Mono<Result> listSendDataBank(@RequestBody @Valid SendDataReq req) {
        return Mono.fromSupplier(() -> apiInterfaceService.getSendDataBankList(req));
    }

}
