package com.jz.dm.controller;

import com.jz.common.utils.Result;
import com.jz.dm.models.dto.DataBankNumberErrorDto;
import com.jz.dm.models.req.DataBankNumberErrorReq;
import com.jz.dm.models.req.LogInfoDetailReq;
import com.jz.dm.models.req.LogInfoListReq;
import com.jz.dm.service.ApiLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.controller
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiLogController
 * @DATE: 2020-12-25/14:25
 * @DAY_NAME_SHORT: 周五
 * @Description:
 **/
@RestController
@RequestMapping("api/logging")
@Api(tags = "日志信息Controller")
public class ApiLogController {

    @Autowired
    private ApiLogService apiLogService;


    @ApiOperation("日志列表查询")
    @PostMapping(value = "/listApiLog")
    public Mono<Result> getApiLogList(@RequestBody @Valid LogInfoListReq req) {
        return Mono.fromSupplier(() -> Result.of_success(apiLogService.listApiLog(req)));
    }

    @ApiOperation("日志详情查询")
    @PostMapping(value = "/getReqDetail")
    public Mono<Result> getApiLogDetail(@RequestBody @Valid LogInfoDetailReq req) {
        return Mono.fromSupplier(() -> apiLogService.getApiLogDetail(req));
    }


    /**
     * 数据银行-查询用户调用数和错误率
     * @param req
     * @return
     */
    @ApiOperation("数据银行-查询用户调用数和错误率")
    @PostMapping(value = "/queryNumberAndError")
    public Result<DataBankNumberErrorDto> queryNumberAndError(@RequestBody DataBankNumberErrorReq req) {
       return apiLogService.queryNumberAndError(req);
    }


}
