package com.jz.dm.controller;

import com.jz.common.utils.Result;
import com.jz.dm.models.req.folder.CreateFolderReq;
import com.jz.dm.models.req.producer.ApiInterfaceReq;
import com.jz.dm.service.ProducerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.controller
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ProducerController
 * @DATE: 2020-12-23/16:44
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@RestController
@RequestMapping("api/producer")
@Api(tags = "Api制作Controller")
public class ProducerController {

    @Autowired
    private ProducerService producerService;

    @ApiOperation("获取文件夹列表")
    @GetMapping(value = "/getFileCatalog")
    public Mono<Result> getFileCatalog(@RequestParam(name = "projectId",required = false) Long projectId,
                                       @RequestParam(name = "orgCode",required = false) String orgCode) {
        return Mono.fromSupplier(() -> producerService.getFileCatalog(projectId,orgCode));
    }

    @ApiOperation("创建项目文件夹")
    @PostMapping(value = "/createProjectFolder")
    public Mono<Result> createProjectFolder(@RequestBody @Valid CreateFolderReq req) {
        return Mono.fromSupplier(() -> producerService.createProjectFolder(req));
    }

    @ApiOperation("服务发布/服务取消")
    @GetMapping(value = "/optionApiIssueStatus")
    public Mono<Result> optionApiIssueStatus(@RequestParam(name = "id") Long id,
                                             @RequestParam(name = "optStatus") Boolean optStatus) {
        return Mono.fromSupplier(() -> producerService.optionApiIssueStatus(id, optStatus));
    }

    @ApiOperation("服务发布---上传")
    @GetMapping(value = "/serverUpload")
    public Mono<Result> serverUpload(@RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> producerService.serverUpload(id));
    }

    @ApiOperation("服务开发---获取ApiId")
    @PostMapping(value = "/getCustomApiId")
    public Mono<Result> getCustomApiId() {
        return Mono.fromSupplier(() -> producerService.getCustomApiId());
    }


    @ApiOperation("Api制作/编辑(第三方)")
    @PostMapping(value = "/addCustomApi")
    public Mono<Result> makeCustomApi(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.saveInterfaceAPi(req)));
    }

    @ApiOperation("Api制作/编辑(数据查询/标签查询)")
    @PostMapping(value = "/addDataTableSelect")
    public Mono<Result> bigDataMakeApi(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> producerService.addBigDataMakeApi(req));
    }
    @ApiOperation("获取function模板列表")
    @GetMapping(value = "/getFunctionTemplateList")
    public Mono<Result> getFunctionTemplateList() {
        return Mono.fromSupplier(() -> producerService.getFunctionTemplateList());
    }

  /*  @ApiOperation("服务发布列表")
    @PostMapping(value = "/getServiceIssueList")
    public Mono<Result> getServiceIssueList(@RequestBody @Valid ServiceIssueReq req) {
        return Mono.fromSupplier(() -> producerService.getServiceIssueList(req));
    }*/
    /**
     * @Description:Api制作(大数据查询/标签查询)保存修改的api信息
     * @return: (自定义更新同时使用)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
   /* @ApiOperation("保存修改的api信息")
    @PostMapping(value = "/saveUpdate")
    public Mono<Result> saveUpdateAPiInfo(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> producerService.saveUpdateAPiInfo(req));
    }*/
    /* *//**
     * @Description:Api制作(数据银行)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     *//*
    @ApiOperation("Api制作(数据银行)")
    @PostMapping(value = "/addDataBankApiInfo")
    public Mono<Result> addDataBankApiInfo(@RequestBody @Valid MakeDataBankApiReq req) {
        return Mono.fromSupplier(() -> producerService.addDataBankApiInfo(req));
    }

    *//**
     * @Description:Api制作(实时接入)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     *//*
    @ApiOperation("Api制作(实时接入)")
    @PostMapping(value = "/realMakeApi")
    public Mono<Result> realCustomApi(@RequestBody @Valid MakeRealCustomApiReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.addRealCustomApi(req)));
    }*/

    /**
     * @Description:保存API基本信息/自定义API
     * @return: (保存API基本信息)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     *//*
    @ApiOperation("保存API基本信息")
    @PostMapping(value = "/saveInterface")
    public Mono<Result> saveInsetInterface(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> producerService.saveInterfaceAPi(req));
    }*/


    /*
     *//**
     * @Description:获取数据源配置
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     *//*
    @ApiOperation("获取数据源配置")
    @PostMapping(value = "/getSourceConfig")
    public Mono<Result> getSourceConfig(@RequestBody @Valid SourceConfigReq req) {
        return Mono.fromSupplier(() -> producerService.getSourceConfig(req));
    }
    *//**
     * @Description:获取索引
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     *//*
    @ApiOperation("获取索引")
    @GetMapping(value = "/getIndicess")
    public Mono<Result> getSourceWarehouseList(@RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> producerService.getSourceWarehouseList(id));
    }
    *//**
     * @Description:获取某个索引下的type
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     *//*
    @ApiOperation("获取某个索引下的type")
    @GetMapping(value = "/getIndicesTypes")
    public Mono<Result> getIndicesTypes(@RequestParam(name = "indices") String indices,
                                        @RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> producerService.getIndicesTypes(indices,id));
    }
    *//**
     * @Description:获取某个索引 type的fields
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     *//*
    @ApiOperation("获取某个索引 type的fields")
    @GetMapping(value = "/getIndicesTypesFields")
    public Mono<Result> getIndicesTypesFields(@RequestParam(name = "indices") String indices,
                                              @RequestParam(name = "type") String type,
                                              @RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> producerService.getIndicesTypesFields(indices,type,id));
    }


    @ApiOperation("保存数据到redis中")
    @GetMapping(value = "/saveRedis")
    public Result saveObjOnRedis() {
        return producerService.saveObjOnRedis();
    }

    @ApiOperation("获取redis中的数据")
    @GetMapping(value = "/getRedisObj")
    public Result getObjOnRedis() {
        return  producerService.getObjOnRedis();
    }*/
}
