package com.jz.dm.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.common.constant.LoggingConstants;
import com.jz.dm.common.util.LogUtil;
import com.jz.dm.gateway.DefaultOpenApiDispatcher;
import com.jz.dm.models.enity.DispatchContext;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 网关路由过滤器
 *
 * @author key
 */
@Component("invokeRouteFilter")
@Slf4j
public class InvokeRouteFilter extends AbstractFilter {

    /** openapi dispatcher logger */
    private static final Logger DISPATCHER_LOGGER = LoggingConstants.DISPATCHER_LOGGER;

    @Autowired
    DefaultOpenApiDispatcher defaultOpenApiDispatcher;

    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_6;
    }

    @Override
    public String getFilterName() {
        return "InvokeRouteFilter";
    }

    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response,
                                    FilterChain chain) {
        String openApiResponse = null;
        long startTime = System.currentTimeMillis();
        try {
            DispatchContext context = new DispatchContext();
            context.setApiKey(request.getApiKey());
            context.setOpenApiMethod(request.getMethod());
            context.setOpenApiParams(request.getParams());
            context.setOpenApiVersion(request.getVersion());
            //扩展参数
            for (GatewayRequest.Attribute attribute : request.getExtAttributes().values()) {
                if (attribute != null && attribute.isPass) {
                    context.setExtAttribute(attribute.name, attribute.value);
                }
            }
            openApiResponse = defaultOpenApiDispatcher.doDispatch(context);
            JSONObject jsonObject = (JSONObject) JSON.parse(openApiResponse);
            response.setAttribute(jsonObject);
        }catch (JSONException ex){
            log.error("json exchange exception =",ex.getMessage());
        }finally {
            long elapseTime = System.currentTimeMillis() - startTime;
            LogUtil.info(DISPATCHER_LOGGER,
                "gateway do default dispatch,request=" + request + ",response=" + openApiResponse
                                            + "," + elapseTime + "ms");
        }
    }


}
