package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.HttpsUtils;
import com.jz.common.utils.Result;
import com.jz.dm.common.enums.GeneralStatusTypeEnum;
import com.jz.dm.common.enums.apiInterface.ApiStatusEnum;
import com.jz.dm.common.enums.auth.AuthModeEnum;
import com.jz.dm.mapper.*;
import com.jz.dm.models.domian.ApiAuth;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.models.domian.ApiInterfaceCustom;
import com.jz.dm.models.dto.ApiServiceApplyDto;
import com.jz.dm.models.req.api.ApiInterfaceDetailReq;
import com.jz.dm.models.req.api.ApiInterfaceInfoListReq;
import com.jz.dm.models.resp.SendDataReq;
import com.jz.dm.service.ApiInterfaceService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiInterfaceServiceImpl
 * @DATE: 2021-1-2/14:27
 * @DAY_NAME_SHORT: 周六
 * @Description:
 **/
@Service("apiInterfaceService")
@Slf4j
public class ApiInterfaceServiceImpl implements ApiInterfaceService {

    /**
     * 测试url
     */
    public static final String testUrl = "";

    @Resource
    private ApiInterfaceMapper apiInterfaceMapper;
    @Resource
    private ApiInterfaceCustomMapper apiInterfaceCustomMapper;
    @Resource
    private ApiAuthMapper apiAuthMapper;
    @Resource
    private ApiInterfaceFileMapper apiInterfaceFileMapper;
    @Resource
    private ApiReqLogMapper apiReqLogMapper;
    @Autowired
    private HttpsUtils httpsUtils;

    /**
     * API---(调用列表)列表查询
     *
     * @param req
     * @return
     */
    @Override
    public IPage<ApiInterface> listCallApiInterface(ApiInterfaceInfoListReq req) {
        IPage<ApiInterface> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getApiKey())) {
            query.eq("ai.api_key", req.getApiKey());
        }
        query.eq("ai.is_deleted", 0);
        query.orderByDesc("ai.create_date");
        query.groupBy("ai.create_date");
        return apiInterfaceMapper.listApiInterfaceInfo(page, query);

    }

    /**
     * API---(未调用列表)列表查询
     *
     * @param req
     * @return
     */
    @Override
    public IPage<ApiInterface> listUnCallApiInterface(ApiInterfaceInfoListReq req) {
        IPage<ApiInterface> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getApiKey())) {
            query.eq("ai.api_key", req.getApiKey());
        }
        query.isNull("ar.api_key");
        query.eq("ai.is_deleted", 0);
        query.orderByDesc("ai.create_date");
        return apiInterfaceMapper.listCallApiInterfaceInfo(page, query);
    }

    /**
     * 服务开发API列表
     *
     * @param req
     * @return
     */
    @Override
    public Result listServerApplyApi(ApiInterfaceInfoListReq req) {
        if (null == req.getProjectId()) {
            return Result.of_error("工程id不能为空!");
        }
        List<Long> listFileIds = null;
        if (null == req.getFileId()) {
            listFileIds = apiInterfaceFileMapper.getProjectFileFolders(req.getProjectId(),null);
        }
        IPage<ApiServiceApplyDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiServiceApplyDto> query = new QueryWrapper<>();
        if (null == req.getFileId()) {
            query.in("ai.file_id", listFileIds);
        } else {
            query.eq("ai.file_id", req.getFileId());
        }
        if (StringUtils.isNotBlank(req.getApiName())){
            query.eq("ai.api_name",req.getApiName());
        }
        if (StringUtils.isNotBlank(req.getStatus())){
            query.eq("ai.status",req.getStatus());
        }
        query.orderByDesc("ai.create_date");
        return Result.of_success(apiInterfaceMapper.listApiDetail(page,query));
    }

    /**
     * 获取API详情信息
     *
     * @param req
     * @return
     */
    @Override
    public Result getApiInterfaceDetail(ApiInterfaceDetailReq req) {
        return Result.of_success(apiInterfaceMapper.selectDetail(req));
    }
    /**
     * api计量统计
     * @param date
     * @return
     */
    @Override
    public Result countAPiCallStat(String date) {
        Date dateParam = null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("YYYY-MM");
            dateParam = dateFormat.parse(date);
        } catch (Exception ex) {
            log.error("时间转换异常:{}", ex.getMessage());
        }
        return Result.of_success(apiReqLogMapper.countApiCallMonthData(dateParam));
    }
    /**
     * 删除API信息
     * @param apiKey
     * @return
     */
    @Override
    public Result delApiInterface(String apiKey) {
        ApiInterface apiInterface = getReqTargetInfo(apiKey);
        if (null == apiInterface) {
            Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        if (ApiStatusEnum.EXPIRY.name().equals(apiInterface.getStatus())) {
            return Result.of_error("API已失效无法再次操作!");
        }
        Integer authStat = getAuthCall(apiInterface.getId(), AuthModeEnum.POWER_CALL_MODE.name(), null);
        Integer authDateStat = getAuthCall(apiInterface.getId(), AuthModeEnum.PERMANENT_TIME_MODE.name(), new Date());
        UpdateWrapper<ApiInterface> updateInfo = new UpdateWrapper<>();
        if (authStat + authDateStat > 0) { //有已授权未使用的先将状态置为$$ 下架
            updateInfo.set("status", ApiStatusEnum.SOLDOUT);//下架状态%%可以调用,但不可以购买
        } else {
            updateInfo.set("status", ApiStatusEnum.EXPIRY);//失效状态
            updateInfo.set("is_deleted", 1);
        }
        updateInfo.eq("id", apiInterface.getId());
        if (apiInterfaceMapper.update(null, updateInfo) > 0) {
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        return Result.of_success(ResultMsg.UPDATE_FAIL);
    }

    /**
     * dmp 删除处理
     *
     * @param apiKey
     * @return
     */
    @Override
    public Result dmpDeleteAuth(String type, String apiKey) {
        ApiInterface apiInterface = getApiInfo(apiKey);
        if (null == apiInterface) {
            return Result.of_error(ResultMsg.DATA_NOT_EXIST);
        }
        if ("0".equals(type)) {//确认是否删除
            Integer call = getAuthCall(apiInterface.getId(), AuthModeEnum.PERMANENT_TIME_MODE.name(), null);
            if (call > 0) {
                return Result.of_error("已存在授权用户,是否要删除!");
            }
            return Result.of_success(ResultMsg.SUCCESS);
        } else if ("1".equals(type)) { //正式删除
            if (ApiStatusEnum.EXPIRY.name().equals(apiInterface.getStatus())) {
                return Result.of_error("API已失效无法再次操作!");
            }
            updateApiAuthStatus(apiInterface.getId());
            UpdateWrapper<ApiInterface> updateWra = new UpdateWrapper<>();
            updateWra.set("status", ApiStatusEnum.EXPIRY);
            updateWra.set("is_deleted", 1);
            updateWra.eq("id", apiInterface.getId());
            if (apiInterfaceMapper.update(null, updateWra) == 0) {
                return Result.of_error(ResultMsg.UPDATE_FAIL);
            }
        }
        return Result.of_success(ResultMsg.UPDATE_SUCCESS);
    }

    /**
     * 更新已经认证的API
     *
     * @param apiInterfaceId
     */
    private void updateApiAuthStatus(Long apiInterfaceId) {
        QueryWrapper<ApiAuth> queryWra = new QueryWrapper<>();
        queryWra.eq("api_interface_id", apiInterfaceId);
        queryWra.eq("handler", 0);//主要是确认单次调用是否调用
        queryWra.eq("auth_mode", AuthModeEnum.PERMANENT_TIME_MODE);
        queryWra.eq("status", GeneralStatusTypeEnum.VALID);
        queryWra.eq("is_deleted", 0);
        List<ApiAuth> authList = apiAuthMapper.selectList(queryWra);
        if (!CollectionUtils.isEmpty(authList)) {
            apiAuthMapper.updateAuthBatch(authList);
        }
    }

    /**
     * 获取单次调用有效授权
     * @param apiId
     * @param modeType
     * @param date
     * @return
     */
    private Integer getAuthCall(Long apiId, String modeType, Date date) {
        //按次调用统计
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("api_interface_id", apiId);
        query.eq("handler", 0);//主要是确认单次调用是否调用
        query.eq("auth_mode", modeType);
        if (null != date) {
            query.gt("valid_end_time", date);
        }
        query.eq("status", GeneralStatusTypeEnum.VALID);
        query.eq("is_deleted", 0);
        return apiAuthMapper.selectCount(query);
    }


    /**
     * 获取有效授权总数
     *
     * @param apiInterface
     * @return
     */
    @Override
    public Integer getValidAuthStatus(ApiInterface apiInterface) {
        Integer authCall = getAuthCall(apiInterface.getId(), AuthModeEnum.POWER_CALL_MODE.name(), null);
        Integer authDateCall = getAuthCall(apiInterface.getId(), AuthModeEnum.PERMANENT_TIME_MODE.name(), new Date());
        return authCall + authDateCall;
    }

    /**
     * 获取api自定义信息
     *
     * @param apiInterfaceId
     * @return
     */
    @Override
    public ApiInterfaceCustom getApiCustomInfo(Long apiInterfaceId) {
        QueryWrapper<ApiInterfaceCustom> queryWra = new QueryWrapper<>();
        queryWra.eq("api_interface_id", apiInterfaceId);
        queryWra.eq("is_deleted", 0);
        return apiInterfaceCustomMapper.selectOne(queryWra);
    }

    /**
     * 获取用户授权信息
     *
     * @param apiKey
     * @param authCode
     * @return
     */
    @Override
    public ApiAuth getApiAuthInfo(String apiKey, String authCode) {
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        query.eq("is_deleted", 0);
        query.eq("api_key", apiKey);
        ApiInterface apiInterface = apiInterfaceMapper.selectOne(query);
        if (null != apiInterface) {
            QueryWrapper<ApiAuth> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("is_deleted", 0);
            queryWrapper.eq("auth_code", authCode);
            ApiAuth apiAuth = apiAuthMapper.selectOne(queryWrapper);
            if (null != apiAuth) {
                return apiAuth;
            }
        }
        return null;
    }

    /**
     * 获取API信息
     *
     * @param apiKey
     * @return
     */
    @Override
    public ApiInterface getApiInfo(String apiKey) {
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        query.eq("is_deleted", 0);
        query.eq("api_key", apiKey);
        return apiInterfaceMapper.selectOne(query);
    }

    @Override
    public ApiInterface getReqTargetInfo(String apiKey) {
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        query.eq("api_key", apiKey);
        query.eq("status", ApiStatusEnum.ISSUE.name());
        query.eq("is_deleted", 0);
        return apiInterfaceMapper.selectOne(query);
    }


    /**
     * 更新APi是否发送到数据银行状态
     *
     * @param apiKey
     * @return
     */
    @Override
    public Result updateApiSendStatus(String apiKey) {
        QueryWrapper<ApiInterface> queryWra = new QueryWrapper<>();
        queryWra.eq("api_key", apiKey);
        queryWra.eq("is_deleted", 0);
        ApiInterface apiInterface = apiInterfaceMapper.selectOne(queryWra);
        if (null == apiInterface) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        UpdateWrapper<ApiInterface> updateWra = new UpdateWrapper<>();
        updateWra.set("is_send_bank", 0);
        updateWra.eq("id",apiInterface.getId());
        if (apiInterfaceMapper.update(null, updateWra) > 0) {
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        return Result.of_error(ResultMsg.UPDATE_FAIL);
    }

    /**
     * @param req
     * @return
     */
    @Override
    public Result getSendDataBankList(SendDataReq req) {
        IPage<ApiInterface> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getApiType())) {
            query.eq("api_type", req.getApiType());
        }
        if (StringUtils.isNotBlank(req.getOutputType())) {
            query.eq("output_type", req.getOutputType());
        }
        if (StringUtils.isNotBlank(req.getStartTime()) && StringUtils.isNotBlank(req.getEndTime())) {
            query.between("create_date", req.getStartTime(), req.getEndTime());
        }
        query.and(wrapper -> wrapper.like("api_name", req.getKey()));
        query.eq("is_send_bank", 1);
        query.eq("status", ApiStatusEnum.ISSUE.name());
        query.eq("is_deleted", 0);
        return Result.of_success(apiInterfaceMapper.selectPage(page, query));
    }
}
