package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.Result;
import com.jz.dm.common.enums.GeneralStatusTypeEnum;
import com.jz.dm.common.enums.apiInterface.ApiStatusEnum;
import com.jz.dm.common.enums.auth.AuthModeEnum;
import com.jz.dm.common.util.DateUtil;
import com.jz.dm.common.util.RandomUtil;
import com.jz.dm.mapper.ApiAuthMapper;
import com.jz.dm.mapper.ApiInterfaceFileMapper;
import com.jz.dm.mapper.ApiInterfaceMapper;
import com.jz.dm.mapper.ApiOrgMapper;
import com.jz.dm.models.domian.ApiAuth;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.models.domian.ApiOrg;
import com.jz.dm.models.dto.ApiServiceApplyDto;
import com.jz.dm.models.req.auth.*;
import com.jz.dm.models.resp.AuthUserResponse;
import com.jz.dm.service.AuthService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

import static com.jz.dm.common.enums.auth.AuthModeEnum.fromTypeModeName;
import static com.jz.dm.common.enums.auth.AuthTypeEnum.fromAuthTypeName;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: AuthServiceImpl
 * @DATE: 2020-12-23/16:48
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("authService")
@Slf4j
public class AuthServiceImpl implements AuthService {


    @Resource
    private ApiAuthMapper apiAuthMapper;
    @Resource
    private ApiInterfaceMapper apiInterfaceMapper;
    @Resource
    private ApiOrgMapper apiOrgMapper;
    @Resource
    private ApiInterfaceFileMapper apiInterfaceFileMapper;

  /*  *//**
     * 认证信息列表
     *
     * @param req
     * @return
     *//*
    @Override
    public Result getAuthListInfo(AuthListInfoReq req) {
        IPage<AuthInfoDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<AuthInfoDto> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getApiKey())) {
            query.eq("ai.api_key", req.getApiKey());
        }
        if (StringUtils.isNotBlank(req.getAuthCode())) {
            query.eq("au.auth_code", req.getAuthCode());
        }
        query.eq("au.is_deleted", 0);
        query.orderByDesc("au.create_date");
        IPage<AuthInfoDto> listSelectApiAuth = apiAuthMapper.listSelectApiAuth(page, query);
        return Result.of_success(listSelectApiAuth);
    }*/

    /**
     * 认证信息详情
     *
     * @param req
     * @return
     */
    @Override
    public Result getAuthDetailInfo(AuthDetailInfoReq req) {
        return Result.of_success(apiAuthMapper.selectAuthDetail(req));
    }

    /**
     * 获取服务授权列表
     * @param req
     * @return
     */
    @Override
    public Result getServiceAuthList(ServiceAuthReq req) {
        List<Long> listFileIds = null;
        if (null == req.getFileId()) {
            listFileIds = apiInterfaceFileMapper.getProjectFileFolders(null,req.getOrgCode());
        }
        IPage<ApiServiceApplyDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiServiceApplyDto> query = new QueryWrapper<>();
        if (null == req.getFileId()) {
            query.in("ai.file_id", listFileIds);
        } else {
            query.eq("ai.file_id", req.getFileId());
        }
        if (StringUtils.isNotBlank(req.getApiName())){
            query.eq("ai.api_name",req.getApiName());
        }
        if (StringUtils.isNotBlank(req.getOrgName())){
            query.like("ar.org_name",req.getOrgName());
        }
        query.eq("ai.is_deleted",0);
        query.orderByDesc("au.create_date");
        return Result.of_success(apiInterfaceMapper.listApiAuthService(page, query));
    }

   /* *//**
     * 添加dmp认证信息
     *
     * @param req
     * @return
     *//*
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addDmpUserApiAuth(AuthUserApiReq req) {
        ApiInterface apiInterface = getInterface(req.getApiKey());
        if (null == apiInterface) {
            return Result.of_error("api授权信息不存在!");
        }
        if (ApiStatusEnum.DRAFT.name().equals(apiInterface.getStatus())){
            return Result.of_error("api未发布,无法授权");
        }
        if () {
            return Result.of_error("api已下架,无法授权");
        }
        ApiOrg apiOrg = apiOrgMapper.selectOne(new QueryWrapper<ApiOrg>().eq("org_code", req.getOrgCode()));
        if (null == apiOrg || !apiOrg.getStatus()) {
            return Result.of_error("组织信息不存在!");
        }

        //授权验证
        if (StringUtils.isNotBlank(req.getAuthType())){
            if (null == fromAuthTypeName(req.getAuthType())){
                return Result.of_error("授权类型不存在!");
            }
        }
        //验证授权方式
        if (StringUtils.isNotBlank(req.getAuthMode())) {
            if (null == fromTypeModeName(req.getAuthMode())) {
                return Result.of_error("授权方式不存在!");
            }
        }
        //认证类型为永久授权
        if (StringUtils.isNotBlank(req.getAuthMode()) &&
                !AuthModeEnum.PERMANENT_TIME_MODE.name().equals(req.getAuthMode())) {
            return Result.of_error("授权类型错误!");
        }
        return saveAuthInfo(apiInterface, apiOrg, req);
    }
*/
    /**
     * 数据银行/DMP用户---授权
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addUserApiAuth(AuthUserApiReq req) {
        ApiInterface apiInterface = getInterface(req.getApiKey());
        if (null == apiInterface) {
            return Result.of_error("api授权信息不存在!");
        }
        if (ApiStatusEnum.SOLDOUT.name().equals(apiInterface.getStatus())
            || ApiStatusEnum.EXPIRY.name().equals(apiInterface.getStatus())) {
            return Result.of_error("api已下架,无法授权");
        }
        if (ApiStatusEnum.DRAFT.name().equals(apiInterface.getStatus())){
            return Result.of_error("api未发布,无法授权");
        }
        ApiOrg apiOrg = apiOrgMapper.selectOne(new QueryWrapper<ApiOrg>().eq("org_code", req.getOrgCode()));
        if (null == apiOrg || !apiOrg.getStatus()) {
            return Result.of_error("组织信息不存在!");
        }
        //授权验证
        if (StringUtils.isNotBlank(req.getAuthType())){
            if (null == fromAuthTypeName(req.getAuthType())){
                return Result.of_error("授权类型不存在!");
            }
        }
        //验证授权方式
        if (StringUtils.isNotBlank(req.getAuthMode())) {
            if (null == fromTypeModeName(req.getAuthMode())) {
                return Result.of_error("授权方式不存在!");
            }
        }
        //认证类型
        if (StringUtils.isNotBlank(req.getAuthMode()) &&
                AuthModeEnum.RECORD_TIME_MODE.name().equals(req.getAuthMode())) {
            if (null == req.getValidStartTime() || null == req.getValidEndTime()) {
                return Result.of_error("开始或结束时间不能为空!");
            }
        }
        return saveAuthInfo(apiInterface, apiOrg, req);
    }

    /**
     * 保存授权信息
     *
     * @param apiInterface
     * @param apiOrg
     * @param req
     * @return
     */
    private Result saveAuthInfo(ApiInterface apiInterface, ApiOrg apiOrg,
                                AuthUserApiReq req) {
        AuthUserResponse response = new AuthUserResponse();
        ApiAuth apiAuth = new ApiAuth();
        BeanUtils.copyProperties(req, apiAuth);
        apiAuth.setApiInterfaceId(apiInterface.getId());
        apiAuth.setApiOrgId(apiOrg.getId());
        apiAuth.setReqType(apiInterface.getLimitType());
        apiAuth.setReqFrequency(apiInterface.getReqFrequency());
        if (!AuthModeEnum.RECORD_TIME_MODE.name().equals(req.getAuthMode())){
            apiAuth.setValidExprDate(DateUtil.getAssignYearDate(99));//过期时间
        }
        //商城只有外部授权
        String authCode = getAuthCode(apiOrg.getOrgCode(), "","");
        //生成盐值
        String salt = RandomUtil.getStringRandom(8);
        apiAuth.setAuthCode(authCode);//授权码
        apiAuth.setSalt(salt);//盐值
        apiAuth.setStatus(GeneralStatusTypeEnum.VALID.name());//授权状态
        response.setAuthCode(authCode);
        response.setSalt(salt);
        if (apiAuthMapper.insert(apiAuth) > 0) {
            return Result.of_success(ResultMsg.SUCCESS, response);
        }
        return Result.of_error(ResultMsg.INSERT_FAIL);
    }


    /**
     * 获取授权码
     *
     * @param type
     * @param userId
     * @param apiOrgCode
     * @return
     */
    private String getAuthCode(String apiOrgCode, String userId, String type) {
        String authCode = "";
        authCode = RandomUtil.generateTokenCode(type, apiOrgCode, userId);
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("auth_code", authCode);
        ApiAuth auth = apiAuthMapper.selectOne(query);
        if (null != auth) {
            authCode = RandomUtil.generateTokenCode(type, apiOrgCode, userId);
        }
        return authCode;
    }

    /**
     * 修改用户认证信息
     *
     * @param req
     * @return
     */
    @Override
    public Result updateAuthInfo(AuthInfoUpdateReq req) {
        ApiAuth apiAuth = apiAuthMapper.selectById(req.getId());
        if (null == apiAuth) {
            return Result.of_error(ResultMsg.DATA_NOT_EXIST);
        }
        UpdateWrapper<ApiAuth> updateWra = new UpdateWrapper<>();
        updateWra.set("is_deleted", 1);
        updateWra.set("status", GeneralStatusTypeEnum.UN_VALID.name());
        updateWra.eq("id", req.getId());
        if (apiAuthMapper.update(null, updateWra) == 0) {
            return Result.of_error(ResultMsg.UPDATE_FAIL);
        }
        return Result.of_success(ResultMsg.UPDATE_SUCCESS);
    }

    /**
     * 盐值重置
     *
     * @param req
     * @return
     */
    @Override
    public Result updateSaltInfo(SaltResetReq req) {
        AuthUserResponse response = new AuthUserResponse();
        ApiAuth apiAuth = getAuthInfo(req.getAuthCode());
        if (null == apiAuth) {
            return Result.of_error("认证信息不存在!");
        }
        String salt = RandomUtil.getStringRandom(8);
        UpdateWrapper<ApiAuth> update = new UpdateWrapper<>();
        update.set("salt", salt);
        update.set("update_date", new Date());
        update.set("update_user", req.getUpdateUser());
        update.eq("id", apiAuth.getId());
        if (apiAuthMapper.update(null, update) > 0) {
            response.setSalt(salt);
            return Result.of_success(ResultMsg.UPDATE_SUCCESS, response);
        }
        return Result.of_success(ResultMsg.UPDATE_FAIL);
    }

    /**
     * 获取API授权信息
     *
     * @param authCode
     * @param apiId
     * @return
     */
    @Override
    public ApiAuth getAuthUser(String authCode, Long apiId) {
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("auth_code", authCode);
        query.eq("status", GeneralStatusTypeEnum.VALID.name());
        query.eq("api_interface_id", apiId);
        query.eq("is_deleted", 0);
        return apiAuthMapper.selectOne(query);
    }

    /**
     * 获取API授权信息
     *
     * @param authCode
     * @return
     */
    @Override
    public ApiAuth getAuthInfo(String authCode) {
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("auth_code", authCode);
        query.eq("status", GeneralStatusTypeEnum.VALID);
        query.eq("is_deleted", 0);
        return apiAuthMapper.selectOne(query);
    }

    /**
     * 更新API状态
     * @param apiAuth
     * @return
     */
    @Override
    public int updateApiAuthStatus(ApiAuth apiAuth) {
        UpdateWrapper<ApiAuth> update = new UpdateWrapper<>();
        update.set("status", GeneralStatusTypeEnum.UN_VALID);
        update.set("handler",1);
        update.set("is_deleted",1);
        update.eq("id",apiAuth.getId());
        return apiAuthMapper.update(null,update);
    }

/*    *//**
     * 模糊查询/
     * @param key
     * @return
     *//*
    @Override
    public Result getAuthNameList(String key) {
        QueryWrapper<ApiOrg> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(key)){
            query.and(wrapper -> wrapper.like("org_name",key)
                    .or().like("org_code",key)
                    .or().like("linkman",key));
        }
        return Result.of_success(apiOrgMapper.selectList(query));
    }*/

    private ApiInterface getInterface(String apiKey) {
        QueryWrapper<ApiInterface> queryInface = new QueryWrapper<>();
        queryInface.last("where is_deleted=0 and api_key ='" + apiKey + "'");
        ApiInterface apiInterface = apiInterfaceMapper.selectOne(queryInface);
        if (null != apiInterface) {
            return apiInterface;
        }
        return null;
    }

    /**
     * 更新授权续费信息
     * @param req
     * @return
     */
    @Override
    public Result updateAuthDate(AuthRenewUpdateReq req) {
        QueryWrapper<ApiAuth> queryWra = new QueryWrapper<>();
        queryWra.eq("auth_code",req.getAuthCode());
        queryWra.eq("is_deleted",0);
        queryWra.eq("status",GeneralStatusTypeEnum.VALID.name());
        ApiAuth apiAuth = apiAuthMapper.selectOne(queryWra);
        if (null == apiAuth){
            return Result.of_error("授权信息不存在!");
        }
        UpdateWrapper<ApiAuth> updateWra = new UpdateWrapper<>();
        updateWra.set("valid_end_time",req.getValidEndTime());
        updateWra.eq("id",apiAuth.getId());
        if (apiAuthMapper.update(null, updateWra) == 0){
            return Result.of_error(ResultMsg.UPDATE_FAIL);
        }
        return Result.of_success(ResultMsg.UPDATE_SUCCESS);
    }
}
