package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.order.bean.dto.OrderDetailDto;
import com.jz.manage.moduls.controller.order.bean.dto.OrderDto;
import com.jz.manage.moduls.controller.order.bean.req.OrderRequest;
import com.jz.manage.moduls.mapper.OrderDao;
import com.jz.manage.moduls.service.OrderService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;

/**
 * 订单表(TOrder)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@Service("orderService")
@Transactional
public class OrderServiceImpl implements OrderService {

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 订单管理列表（分页查询）
     * @param req
     * @param httpServletRequest
     * @return
     * @throws Exception
     */
    @Override
    public Result<IPage<OrderDto>> queryListOrderInfo(OrderRequest req, HttpServletRequest httpServletRequest) throws Exception {
        IPage<OrderDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<OrderDto> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getSearch())) {
            query.and(wrapper -> wrapper.like("t2.data_name", req.getSearch())
                    .or().like("t4.department_name", req.getSearch()));
        }
        if (StringUtils.isNotBlank(req.getPurchaseMethod())) {
            query.eq("t.purchaseMethod", req.getPurchaseMethod());
        }

        if (StringUtils.isNotBlank(req.getOrderTime())) {
            String startTime = req.getOrderTime().substring(0, 10) + " 00:00:00";
            String endTime = req.getOrderTime().substring(11, 21) + " 23:59:59";
            query.between("t.order_time", startTime, endTime);

        }
        query.eq("t.del_flag", "N");
        query.orderByDesc("t.order_time");
        IPage<OrderDto> list = orderDao.queryListOrderInfo(page, query);
        if (list.getTotal() == 0) {
            return Result.of_error("没有订单！");
        }
        return Result.of_success(list);
    }

    /**
     * 根据订单主键查询，订单详情
     * @param orderId
     * @return
     * @throws Exception
     */
    @Override
    public OrderDetailDto queryOrderDetail(String orderId) throws Exception {
        return orderDao.queryOrderDetail(orderId);
    }
}